#ifdef __ia64__
#define __ASSEMBLY__
#include <asm/unistd.h>
	.text
	.align 32
	.globl	__ia64_syscall
	.proc	__ia64_syscall
__ia64_syscall:
	.regstk 6,0,0,0
	mov r15=in5
	break __BREAK_SYSCALL
	cmp.eq p6,p0=-1,r10
	;;
(p6)	sub r8=0,r8
	br.ret.sptk.many rp
	.endp __ia64_syscall
#endif
